#!/bin/sh

mem='0'
# first try to detect physical memory
# dmidecode is only present on x86/x64 systems
if which dmidecode > /dev/null; then
  mem=$(dmidecode -t memory | awk '$1 == "Size:" { sum += $2 } END { print sum }')
  mem=$((mem * 0x100000))
fi

# otherwise use amount of os memory
if [ "$mem" = "0" ]; then
  mem=`sed -nr 's/^MemTotal:[ \t]+([0-9]+).*/\1/p' /proc/meminfo`
  mem=$(($mem * 1024))
fi

swap=`sed -nr 's/^SwapTotal:[ \t]+([0-9]+).*/\1/p' /proc/meminfo`
swap=$(($swap * 1024))
kernel=$(cut -d' ' -f3 /proc/version)

os="Linux"
if [ -e /etc/os-release ]; then
  os=$(grep ^PRETTY_NAME= /etc/os-release | cut -d= -f2)
elif [ -e /etc/lsb-release ]; then
  os=$(grep ^DISTRIB_DESCRIPTION= /etc/lsb-release | cut -d= -f2)
fi
os=${os#\"}
os=${os%\"}
os="$os (kernel $kernel)"

# get libc version
if [ -e /lib/libuClibc-*.so ]; then
  libc=$(ls /lib/libuClibc-*.so | head -n1)
  libc=${libc#*-}
  libc=${libc%.so}
  libc="uClibc $libc"
elif [ -e /lib/libc-*.so ]; then
  libc=$(ls /lib/libc-*.so | head -n1)
  libc=${libc##*-}
  libc=${libc%.so}
  libc="glibc $libc"
elif [ -L /lib/ld-linux.so.* ]; then
  libc=$(readlink /lib/ld-linux.so.*)
  libc=${libc##*-}
  libc=${libc%.so}
  libc="glibc $libc"
fi

if [ -n "$libc" ]; then
  os="$os ($libc)"
fi
if uname -m | grep -q 64; then
  os="$os 64-bit"
else
  os="$os 32-bit"
fi

dmi="/sys/devices/virtual/dmi/id"
if [ -e "$dmi" ]; then
  vendor=$(cat "$dmi/board_vendor")
  product=$(cat "$dmi/board_name")
  fwvendor=$(cat "$dmi/bios_vendor")
  fwver=$(cat "$dmi/bios_version")
elif grep -q poeinfo /proc/cpuinfo; then
  vendor="Exacq Technologies"
  poe_version=$(i2c 30 3)
  npoe=$((4 << (poe_version & 3)))
  product=$(printf "PE%02d-M" $npoe)
  fwver=$(cat /etc/version)
elif which parhandclient > /dev/null; then
  vendor="Axis"
  product=$(parhandclient get root.Brand.ProdFullName - raw 2> /dev/null)
  product=${product#AXIS }
  fwver=$(parhandclient get root.Properties.Firmware.Version - raw 2> /dev/null)
elif [ -e /var/info/sys/brand.xml ]; then
  vendor="Illustra"
  product=$(xml sel -t -v /brand/product/fullname /var/info/sys/brand.xml)
  product=${product#Illustra }
  fwver=$(xml sel -t -v /brand/version/firmware /var/info/sys/brand.xml)
elif [ -e /opt/ad/srv/www/htdocs/data/MODELCACHE.conf ]; then
  vendor="Illustra"
  . /opt/ad/srv/www/htdocs/data/MODELCACHE.conf
  product="$PRODUCTCODE"
  fwver="$FIRMWAREVERSION"
elif [ -e /etc/firmware-version ]; then
  vendor="Vivotek"
  product=$(cut -d- -f1 /etc/firmware-version)
  fwver=$(cut -d- -f3 /etc/firmware-version)
fi

if [ -z "$fwvendor" ]; then
  fwvendor="$vendor"
fi

# get SOC 
if grep -q -i artpec-3 /proc/cpuinfo; then
  soc="Axis artpec-3"
  proc="crisv32"
elif grep -q -i artpec-[45] /proc/cpuinfo; then
  if grep -q -i artpec-4 /proc/cpuinfo; then
    soc="Axis artpec-4"
  else
    soc="Axis artpec-5"
  fi
  proc=$(grep "^cpu model" /proc/cpuinfo | head -n1)
  proc=${proc#*: }
else
  proc=$(grep -m1 ^Processor /proc/cpuinfo)
  proc=${proc#*: }
  if [ -z "$proc" ]; then
    proc=$(grep -m1 "^model name" /proc/cpuinfo)
    proc=${proc#*: }
  fi
  soc=$(grep -m1 ^Hardware /proc/cpuinfo)
  soc=${soc#*: }
  case "$soc" in
    hi3516a) soc="HiSilicon hi3516a"; proc="ARM Cortex-A7";;
    hi3535) soc="HiSilicon hi3535"; proc="ARM Cortex-A9";;
    Axis|Coconut|Condor) soc="Ambarella A5S";;
    Two-Face) soc="Ambarella S2-88"; proc="ARM Cortex-A9";;
    "Ambarella S2L"*) soc="Ambarella S2L"; proc="ARM Cortex-A9";;
    adillustra610) soc="Texas Instruments TI8148"; proc="ARM Cortex-A8";;
    DaVinci*) soc="Texas Instruments $soc";;
    *Xarina*) soc="Sony Xarina";;
  esac
  if [ -n "$soc" ]; then
    proc="$proc ($soc)"
  fi
fi

cat <<EOF
motherboard-vendor=$vendor
motherboard-model=$product
firmware-vendor=$fwvendor
firmware-version=$fwver
memory=$mem
swap=$swap
processor=$proc
os=$os
EOF

# Get versions of exacq's deploy key and base image if found.
deploykey_version=$(grep -oP "^Key Name: \K.*" /versions.txt 2>/dev/null)
if [ $? -eq 0 ] && [ ! -z "$deploykey_version" ]; then
    echo "deploykey-version=$deploykey_version"
fi
deploykey_jobnumber=$(grep -oP "^Job: \K[0-9]*" /versions.txt 2>/dev/null)
if [ $? -eq 0 ] && [ ! -z "$deploykey_jobnumber" ]; then
    echo "deploykey-jobnumber=$deploykey_jobnumber"
fi
deploykey_branchname=$(grep -oP "^Branch: \K.*" /versions.txt 2>/dev/null)
if [ $? -eq 0 ] && [ ! -z "$deploykey_branchname" ]; then
    echo "deploykey-branchname=$deploykey_branchname"
fi
# Note that the release notes pathname varies between OPS and RETAIL keys.
baseimage_version=$(grep -oP "Linux-1804/Release_Notes.txt:\K.*" /versions.txt 2>/dev/null)
if [ $? -eq 0 ] && [ ! -z "$baseimage_version" ]; then
    echo "baseimage-version=$baseimage_version"
fi

# #15756 
#
# For SIIG, scsi ids look likst scsi@B:D1.D2.D3
# where B: busid, and Dx are device id fields. B is unique and Dx are all 0's 
#
# For LSI, busid is always 0. but D2 field is unique. 
# Offsetting the d2 value on LSI by 99, that gives indices in the range 99->107. 
# Also offset the 'b' by hundred as well. This ensures that the OS drive falls out of range and is not displayed in the gui. 
# So final formula for lsi hba is index = (b * 100) + d2 + 99
# Use this value in hwmconfig.xml drive ordering
#
# This formula ensures existing SIIG ordering is unchanged because all the device ids are 0's. So if scsiid for a SIIG device is 2:0.0.0, index will still be 2. 
# Also ensures that on a system with LSI, the unique index does not collide with SIIG index except index 0.

rm -f sysinfo.sh.log
echo "=============================================" >> sysinfo.sh.log

for edvrmountpt in $(ls /mnt/edvr); do
  # In 18.04, the exacq image configures all RAID and non-RAID drives with x-systemd.automount
  # which permits server to startup without waiting for mounts to complete. Hence this may
  # execute before recording drives mounted, and therefore may not see expected partitions and
  # therefore indicate false positive "needs prep". The easy cure for this is to just try
  # to access the output file from PSFPI's read-write test. Swallow any errors, because this
  # is just meant to prod automount just in case.
  cat "/mnt/edvr/$edvrmountpt/psfpi/test.bin" > /dev/null 2>&1
done

has_lsi_hba=
if which lspci > /dev/null && lspci | grep -q 'LSI Logic / Symbios Logic SAS3008'; then
  has_lsi_hba=1
fi

for dev in $(ls /sys/block/ | grep ^sd); do
  ls /sys/block/$dev/device/scsi_device/ | {
    IFS=: read b d1 d2 d3
    index=$b
    if [ "$has_lsi_hba" = "1" ]; then
      index=$((b * 100 + d2 + 99))
    fi
    echo "drive-index=$index"
    echo "drive-index=$index" >> sysinfo.sh.log
  }
  udevadm info --query=property --name=/dev/$dev | {
    serial=
    while IFS='=' read x1 x2; do
      case "$x1" in
        ID_SCSI_SERIAL)
          serial=$x2
          ;;
        ID_SERIAL_*)
          if [ -z "$serial" ]; then
            serial=$x2
          fi
          ;;
      esac
    done
    echo "drive-serial=$serial"
    echo "drive-serial=$serial" >> sysinfo.sh.log
  }
  removable=$(cat /sys/block/$dev/removable)
  fspath=
  fstype=
  system=0
  numparts=0
  while read x1 x2 x3 x4; do
    if [ "/dev/$dev" = "${x1%%[0-9]*}" ]; then
      # psfpi expects trailing /
      fspath=${x2%/}/
      fstype=$x3
      if [ "$fspath" = "/" -o "$fspath" = "/mnt/export/" -o "$fstype" = "swap" ]; then
        system=1
      fi
      numparts=$((numparts + 1))
    fi
  done < /etc/mtab
  echo "drive-name=$dev"
  echo "drive-system=$system"
  echo "drive-partitions=$numparts"
  echo "drive-removable=$removable"
  echo "drive-fspath=$fspath"
  echo "drive-fstype=$fstype"


  echo "drive-name=$dev" >> sysinfo.sh.log
  echo "drive-system=$system" >> sysinfo.sh.log
  echo "drive-partitions=$numparts" >> sysinfo.sh.log
  echo "drive-removable=$removable" >> sysinfo.sh.log
  echo "drive-fspath=$fspath" >> sysinfo.sh.log
  echo "drive-fstype=$fstype" >> sysinfo.sh.log
done
